"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isGetStaticProps = isGetStaticProps;
exports.default = transformNextGetStaticPropsWithStaticContext;
exports.EXPORT_NAME_GET_STATIC_PROPS = void 0;

/* eslint-disable no-nested-ternary */
const EXPORT_NAME_GET_STATIC_PROPS = 'getStaticProps';
exports.EXPORT_NAME_GET_STATIC_PROPS = EXPORT_NAME_GET_STATIC_PROPS;
const ssgExports = new Set([EXPORT_NAME_GET_STATIC_PROPS, // legacy methods added so build doesn't fail from importing
// server-side only methods
`unstable_getStaticProps`]);

function isGetStaticProps(node) {
  switch (node.type) {
    case 'FunctionDeclaration':
      {
        return !!node.id && ssgExports.has(node.id.name);
      }

    case 'VariableDeclarator':
      {
        const lval = node.id;

        if (lval.type === 'Identifier') {
          return ssgExports.has(lval.name);
        }

        return false;
      }

    default:
      return false;
  }
}

function transformNextGetStaticPropsWithStaticContext({
  types: t
}) {
  function transform(exportNamedPath, needsImport, params, body, async) {
    const originalGetStaticProps = t.functionDeclaration(t.identifier('_getStaticProps'), params, body, false, async);
    const newBody = t.blockStatement([t.variableDeclaration('const', [t.variableDeclarator(t.identifier('originalStaticProps'), async ? t.awaitExpression(t.callExpression(t.identifier('_getStaticProps'), [t.spreadElement(t.identifier('args'))])) : t.callExpression(t.identifier('_getStaticProps'), [t.spreadElement(t.identifier('args'))]))]), t.variableDeclaration('const', [t.variableDeclarator(t.identifier('staticContext'), t.awaitExpression(t.callExpression(t.identifier('buildStaticContextValue'), [])))]), t.returnStatement(t.objectExpression([t.spreadElement(t.identifier('originalStaticProps')), t.objectProperty(t.identifier('props'), t.objectExpression([t.spreadElement(t.memberExpression(t.identifier('originalStaticProps'), t.identifier('props'))), t.objectProperty(t.identifier('staticContext'), t.identifier('staticContext'), false, true)]))]))]);
    const newParams = [t.restElement(t.identifier('args'))];
    const newGetStaticProps = t.functionDeclaration(t.identifier('getStaticProps'), newParams, newBody, false, true);
    const newExport = t.exportNamedDeclaration(newGetStaticProps);
    const getStaticContextImport = t.importDeclaration([t.importSpecifier(t.identifier('buildStaticContextValue'), t.identifier('buildStaticContextValue'))], t.stringLiteral('next-static-context'));

    if (needsImport) {
      exportNamedPath.parentPath.unshiftContainer('body', getStaticContextImport);
    }

    exportNamedPath.remove();
    exportNamedPath.parentPath.pushContainer('body', originalGetStaticProps);
    exportNamedPath.parentPath.pushContainer('body', newExport);
  }

  return {
    visitor: {
      Program: {
        enter(path, state) {
          state.done = false;
          state.needsImport = true;
          path.traverse({
            ImportDeclaration(importDeclarationPath, importDeclarationState) {
              if (importDeclarationPath.node.source.value === 'next-static-context' && importDeclarationPath.node.specifiers.some(specifier => t.isImportSpecifier(specifier) && t.isIdentifier(specifier.local) && specifier.local.name === 'buildStaticContextValue')) {
                importDeclarationState.needsImport = false;
              }
            }

          }, state);
          path.traverse({
            ExportNamedDeclaration(exportNamedPath, exportNamedState) {
              const {
                declaration
              } = exportNamedPath.node;

              if (exportNamedState.done) {
                return;
              }

              if (t.isFunctionDeclaration(declaration)) {
                if (!isGetStaticProps(declaration)) {
                  return;
                }

                if (exportNamedState.done) {
                  return;
                }

                exportNamedState.done = true;
                const {
                  params,
                  body,
                  async
                } = declaration;
                transform(exportNamedPath, exportNamedState.needsImport, params, body, async);
                return;
              }

              if (t.isVariableDeclaration(declaration)) {
                const functionDeclarator = declaration.declarations.find(declarator => t.isFunctionExpression(declarator.init) || t.isArrowFunctionExpression(declarator.init));

                if (!functionDeclarator || !isGetStaticProps(functionDeclarator)) {
                  return;
                }

                if (exportNamedState.done) {
                  return;
                }

                exportNamedState.done = true;
                const functionExpression = functionDeclarator.init;
                const {
                  params,
                  body,
                  async
                } = functionExpression;
                const functionBody = t.isExpression(body) ? t.blockStatement([t.returnStatement(body)]) : body;
                transform(exportNamedPath, exportNamedState.needsImport, params, functionBody, async);
              }
            }

          }, state);
        }

      }
    }
  };
}