"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.withStaticContext = withStaticContext;

var _path = require("path");

function isRuleSetUseItem(useItem) {
  return !!useItem.loader;
}
/** Returns a reference to the webpack rule for the babel-loader */


function findNextBabelRule(config) {
  var _config$module, _config$module$rules;

  return (_config$module = config.module) === null || _config$module === void 0 ? void 0 : (_config$module$rules = _config$module.rules) === null || _config$module$rules === void 0 ? void 0 : _config$module$rules.find(rule => {
    var _rule$use, _rule$use2, _rule$use2$loader;

    if (typeof rule === 'string' || typeof rule.use === 'string' || typeof rule.use === 'function') {
      return false;
    }

    if (Array.isArray(rule.use)) {
      return rule.use.some(useItem => {
        if (isRuleSetUseItem(useItem)) {
          var _useItem$loader;

          return useItem.loader === 'next-babel-loader' || ((_useItem$loader = useItem.loader) === null || _useItem$loader === void 0 ? void 0 : _useItem$loader.includes('babel/loader'));
        }

        return false;
      });
    }

    return ((_rule$use = rule.use) === null || _rule$use === void 0 ? void 0 : _rule$use.loader) === 'next-babel-loader' || ((_rule$use2 = rule.use) === null || _rule$use2 === void 0 ? void 0 : (_rule$use2$loader = _rule$use2.loader) === null || _rule$use2$loader === void 0 ? void 0 : _rule$use2$loader.includes('babel/loader'));
  });
}
/** Modifies the next.js config as needed to add support for this plugin */


function withStaticContext(componentsDir, nextConfig = {}) {
  return { ...nextConfig,
    env: {
      NEXT_STATIC_CONTEXT_REQUIRE_CONTEXT: (0, _path.resolve)(process.cwd(), componentsDir),
      ...nextConfig.env
    },

    webpack(config, options) {
      const {
        isServer
      } = options;
      const nextBabelRule = findNextBabelRule(config);

      if (!nextBabelRule || typeof nextBabelRule === 'string') {
        return config;
      }

      const babelLoader = require.resolve('./webpack/next-static-context-babel-loader'); // Update the babel rule to run our babel loader first


      if (Array.isArray(nextBabelRule.use)) {
        nextBabelRule.use.unshift({
          loader: babelLoader,
          options: {
            isServer
          }
        });
      } else {
        nextBabelRule.use = [{
          loader: babelLoader,
          options: {
            isServer
          }
        }, nextBabelRule.use];
      } // Update the externals config to count our plugin as _internal_.
      // This forces webpack to bundle our plugin, which is necessary to
      // support our use of `require.context()`.


      if (isServer) {
        const nextBaseExternals = config.externals;
        config.externals = [async ({
          context,
          request,
          dependencyType,
          getResolve
        }) => {
          const isNextStaticContext = /^next-static-context/;

          if (request && isNextStaticContext.test(request)) {
            return undefined;
          }

          return nextBaseExternals[0]({
            context,
            request,
            dependencyType,
            getResolve
          });
        }];
      } // If there's a user-defined webpack config function,
      // be sure to call that, too.


      if (typeof nextConfig.webpack === 'function') {
        return nextConfig.webpack(config, options);
      }

      return config;
    }

  };
}